==> GFS-4.1.0.tar.gz: GFS-4.1.0/src/tools/gfsck/pass1.c <==
/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-2000 Regents of the University of Minnesota
**    Portion Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@sistina.com>
**
**    Module name: pass1.c  (gfsck)
**
**    Description:
**      Filesystem checker
**
*******************************************************************************
******************************************************************************/

#define _LARGEFILE64_SOURCE
#include "gfs.h"
#include "gfsck.h"
#include "recover.h"

#include "dio.h"
#include "blklist.h"
#include "bits.h"
#include "gfs_ondisk.h"



/**
 * pass_1
 * @sdp: superblock
 *
 */

int pass_1(gfs_sbd_t *sdp)
{
  gfs_rgrpd_t *rgd;
  osi_list_head_t *tmp;
  uint32 start_blk, start_buf, bsize = sdp->sd_sb.sb_bsize;
  uint64 block;
  char buf[sdp->sd_sb.sb_bsize];
  int error, cnt = 0;  
  gfs_meta_header_t mheader, *mh = &mheader;
  gfs_dinode_t di;
  int count = 0, first;

  
  for (tmp = sdp->sd_rglist.next; tmp != &sdp->sd_rglist; tmp = tmp->next)
  {
    printf("Pass 1:  %.0f %% \r", 100.0 * ((float)cnt / (float)rg_count));
    fflush(0);
    cnt++;

    rgd = osi_list_entry(tmp, gfs_rgrpd_t, rd_list);

    error = gfs_rgrp_read(rgd);
    if (error)
      return -1;
    
    start_blk = 0;
    start_buf = 0;
    first = 1;

    while (1)
    {
      error = next_rg_meta(rgd, &start_blk, &start_buf, &block, &first);
      if (error)
	break;

      first = 0;

      memset(buf, 0, bsize);

      if (do_lseek(fsfd, block * bsize))
      {
	warn("Bad lseek (#%"FMT_64"u)\n", block);
	return -1;
      }
      
      if (do_read(fsfd, &buf, bsize))
      {
	warn("Bad read (#%"FMT_64"u)\n", block);
	return -1;
      }

      gfs_meta_header_in(mh, buf);

      if (mh->mh_magic != GFS_MAGIC)
      {
	warn("Bad meta magic (#%"FMT_64"u)\n", block);
	continue;
      }

      if (mh->mh_type != GFS_METATYPE_DI &&
	  mh->mh_type != GFS_METATYPE_IN &&
	  mh->mh_type != GFS_METATYPE_LF &&
	  mh->mh_type != GFS_METATYPE_JD)
      {
	warn("Bad meta type (#%"FMT_64"u)\n", block);
	continue;
      }

      if (mh->mh_type != GFS_METATYPE_DI)
	continue;

      /* dinode checks */

      gfs_dinode_in(&di, buf);

      if (di.di_flags & GFS_DIF_UNUSED)
	continue;

      count++;

      if (di.di_num.no_addr != block)
      {
	warn("Bad dinode num addr (#%"FMT_64"u)\n", di.di_num.no_addr);
	continue;
      }

      if (di.di_rgrp != rgd->rd_ri.ri_addr && (block > sdp->sd_sb.sb_root_di.no_addr))
      {
	warn("Bad dinode rgrp number (#%"FMT_64"u)\n", di.di_num.no_addr);
	continue;
      }

      if (di.di_type != FILE_REG && di.di_type != FILE_DIR &&
	  di.di_type != FILE_LNK && di.di_type != FILE_BLK &&
	  di.di_type != FILE_CHR && di.di_type != FILE_FIFO &&
	  di.di_type != FILE_SOCK)
      {
	warn("Bad dinode type (#%"FMT_64"u)\n", di.di_num.no_addr);
	continue;
      }

      if (di.di_height != compute_height(sdp, di.di_size))
      {
	warn("Bad dinode height (#%"FMT_64"u)\n", di.di_num.no_addr);
	continue;
      }

      if (di.di_type == FILE_DIR && (di.di_flags & GFS_DIF_EXHASH))
      {
	if ((1 << di.di_depth) * sizeof(uint64) != di.di_size)
	{
	  warn("Bad dir dinode depth (#%"FMT_64"u)\n", di.di_num.no_addr);
	  continue;
	}
      }
    }
       
    gfs_rgrp_relse(rgd);
  }

  printf("Pass 1:  %d dinodes found\n", count);
  
  return 0;
}

==> GFS-4.1.1.tar.gz: GFS-4.1.1/src/tools/gfsck/pass1.c <==
/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-2000 Regents of the University of Minnesota
**    Portion Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@sistina.com>
**
**    Module name: pass1.c  (gfsck)
**
**    Description:
**      Filesystem checker
**
*******************************************************************************
******************************************************************************/

#include "gfs.h"
#include "gfsck.h"
#include "recover.h"

#include "dio.h"
#include "blklist.h"
#include "bits.h"
#include "gfs_ondisk.h"



/**
 * pass_1
 * @sdp: superblock
 *
 */

int pass_1(gfs_sbd_t *sdp)
{
  gfs_rgrpd_t *rgd;
  osi_list_head_t *tmp;
  uint32 start_blk, start_buf, bsize = sdp->sd_sb.sb_bsize;
  uint64 block;
  char buf[sdp->sd_sb.sb_bsize];
  int error, cnt = 0;  
  gfs_meta_header_t mheader, *mh = &mheader;
  gfs_dinode_t di;
  int count = 0, first;

  
  for (tmp = sdp->sd_rglist.next; tmp != &sdp->sd_rglist; tmp = tmp->next)
  {
    printf("Pass 1:  %.0f %% \r", 100.0 * ((float)cnt / (float)rg_count));
    fflush(0);
    cnt++;

    rgd = osi_list_entry(tmp, gfs_rgrpd_t, rd_list);

    error = gfs_rgrp_read(rgd);
    if (error)
      return -1;
    
    start_blk = 0;
    start_buf = 0;
    first = 1;

    while (1)
    {
      error = next_rg_meta(rgd, &start_blk, &start_buf, &block, &first);
      if (error)
	break;

      first = 0;

      memset(buf, 0, bsize);

      if (do_lseek(fsfd, block * bsize))
      {
	warn("Bad lseek (#%"FMT_64"u)\n", block);
	return -1;
      }
      
      if (do_read(fsfd, &buf, bsize))
      {
	warn("Bad read (#%"FMT_64"u)\n", block);
	return -1;
      }

      gfs_meta_header_in(mh, buf);

      if (mh->mh_magic != GFS_MAGIC)
      {
	warn("Bad meta magic (#%"FMT_64"u)\n", block);
	continue;
      }

      if (mh->mh_type != GFS_METATYPE_DI &&
	  mh->mh_type != GFS_METATYPE_IN &&
	  mh->mh_type != GFS_METATYPE_LF &&
	  mh->mh_type != GFS_METATYPE_JD)
      {
	warn("Bad meta type (#%"FMT_64"u)\n", block);
	continue;
      }

      if (mh->mh_type != GFS_METATYPE_DI)
	continue;

      /* dinode checks */

      gfs_dinode_in(&di, buf);

      if (di.di_flags & GFS_DIF_UNUSED)
	continue;

      count++;

      if (di.di_num.no_addr != block)
      {
	warn("Bad dinode num addr (#%"FMT_64"u)\n", di.di_num.no_addr);
	continue;
      }

      if (di.di_rgrp != rgd->rd_ri.ri_addr && (block > sdp->sd_sb.sb_root_di.no_addr))
      {
	warn("Bad dinode rgrp number (#%"FMT_64"u)\n", di.di_num.no_addr);
	continue;
      }

      if (di.di_type != FILE_REG && di.di_type != FILE_DIR &&
	  di.di_type != FILE_LNK && di.di_type != FILE_BLK &&
	  di.di_type != FILE_CHR && di.di_type != FILE_FIFO &&
	  di.di_type != FILE_SOCK)
      {
	warn("Bad dinode type (#%"FMT_64"u)\n", di.di_num.no_addr);
	continue;
      }

      if (di.di_height < compute_height(sdp, di.di_size))
      {
	warn("Bad dinode height (#%"FMT_64"u)\n", di.di_num.no_addr);
	continue;
      }

      if (di.di_type == FILE_DIR && (di.di_flags & GFS_DIF_EXHASH))
      {
	if ((1 << di.di_depth) * sizeof(uint64) != di.di_size)
	{
	  warn("Bad dir dinode depth (#%"FMT_64"u)\n", di.di_num.no_addr);
	  continue;
	}
      }
    }
       
    gfs_rgrp_relse(rgd);
  }

  printf("Pass 1:  %d dinodes found\n", count);
  
  return 0;
}
